/**************************************************************************************
* MS5534A BArometer Library for PIC uC
* 05.04.18 GJO. Current working version
* Purpose: Provides all required interface functions to control
*          an Intersems MS5534A sensor
* Target:	PIC16F877, PIC18F452 or similar
* Note:	The following functions were ported from the Intersema
*			Programming application note using a 8051 type target.
*			Some work will be required to make it work in your system.
*			But all the important functions are provided here.
* 			The output is in raw form and will require additional
*			programming to linearize and temperature compensate the
*			readings. Please read all Intersema documentation
*			BE very careful soldering the device and follow all
*			Intersema recommendation to avoid destroying the sensor.
*
*	MS5534 Wiring:
*	-------------------------------------------
*	PIC		MS5534A
*	-------------------------------------------
*	SCK		SCK
*	SDI		SDO
*	SDO		SDI
*	PIN_B4	(Connects to my interface logic as there are more SPI devices in my system)
***************************************************************************************/

// Note: Further program setup in the target will be required

#define SDI                         PIN_C4  // Input
#define SDO                         PIN_C5  // Output
#define SCK                         PIN_C3  // Output
#define MS5534_CS                   PIN_B4  // MS5534 chip select

// MS5534 Support
#separate void read_barometer();
#separate void SerialSendLsbFirst(int, int);
#separate long SerialGet16 (void);
#separate void reset_MS5534A(void);
#separate long get_data(int);
#separate long getW (int);
#separate long waitOnDoutFall (void);
#separate long ConvertWtoC5534 (int, long, long, long, long);

/******************************************************************************
* Function:     read_barometer()
* Invoked:      RS-232 Command processor
* Description:  Reads all MS5534A data registers and prints it to RS-232
*
* Parameters:   None
* Return Val:   None
* Variables:
* Constants:
* Flags:
* Interrupts:   None
*******************************************************************************/
#separate
void read_barometer()
{
    long cw[4] = 0,0,0,0;
    long fc[6] = 0,0,0,0,0,0;
    signed long d1 = -999, d2 = -999;
    int i;

    setup_spi(SPI_SS_DISABLED);     // Disable SPI sub system to release pin access
    output_high(MS5534_CS);         // Enable MS5534 Device Select
    reset_MS5534A();                // Initialize MS5534A

    for (i=0; i<4; i++)             // Get MS5534 calibration words
            cw[i] = getW(i);

    if(action[1] == '1')            // If U1B1 is sent, then print only cal data
    {
        for (i=0; i<6; i++)             // Extract cal coefficients
        {
            fc[i] = ConvertWtoC5534 (i, cw[0], cw[1], cw[2], cw[3]);
            printf("%lu ", fc[i]);
        }
    }
    else
    {
        if ((cw[0] != 0) & (cw[0] != 0xFFFF))   // Check for valid sensor
        {                                       //   otherwise return error defaults
            d1 = get_Data(PRESSURE);            // read pressure value
            d2 = get_Data(TEMPERATURE);         // read temperature value
        }
        printf ("%ld %ld ", d1, d2);
    }
    output_low(MS5534_CS);
    setup_spi(SPI_MASTER | SPI_H_TO_L | SPI_CLK_DIV_64);
}

//******************************************************************************
//* reset_MS5534A
//******************************************************************************
//
//  This function performs a unconditional MS5534A system reset, which is accepted
//  at any time by the MS5534.
//  Reset requires 21 clock pulses with sending 2 bytes with alternating bytes
//  such as 0x55 0x55 0x00

#separate
void reset_MS5534A(void)
{
    SerialSendLsbFirst (0x55, 8);   //  21-bit reset sequence 0x55 0x55 0x00
    SerialSendLsbFirst (0x55, 8);
    SerialSendLsbFirst (0x00, 5);
}

//******************************************************************************
//* getW ( get MS5534A calibration coefficients words )
//******************************************************************************
#separate
long getW (int index)
{
    int cmd1 = 0, cmd2 = 0;
    long wdata;
    wdata = 0;

    switch(index)
    {
        case 0:
            cmd1 = 0x57;
            cmd2 = 0x01;
            break;
        case 1:
            cmd1 = 0xD7;
            break;
        case 2:
            cmd1 = 0x37;
            cmd2 = 0x01;
            break;
        case 3:
            cmd1 = 0xB7;
            break;
    }
    SerialSendLsbFirst (cmd1, 8);
    SerialSendLsbFirst (cmd2, 5);
    wdata = SerialGet16();
    SerialSendLsbFirst (0x00, 1);

    return(wdata);
}

//******************************************************************************
//*         SerialSendLsbFirst (send commands to MS5534A)
//******************************************************************************
#separate
void SerialSendLsbFirst (int pattern, int nbr_clock)
{
    int i;
    int c;
    output_low(SCK);
    delay_us(1);

    for (i=0; i < nbr_clock; i++)
    {
        c = pattern & 0x01;
        if (c==1)
            output_high(SDO);
        else
            output_low(SDO);
        delay_us(1);
        output_high(SCK);
        delay_us(1);
        output_low(SCK);
        pattern = pattern >> 1;
    }
}

//******************************************************************************
//*     long SerialGet16 (void) (MS5534A receive data function)
//******************************************************************************
#separate
long SerialGet16 (void)
{
    int j;
    long v;
    v = 0;
    output_low(SCK);
    delay_us(2);

    for (j=0; j < 16; j++)
    {
        output_high(SCK);
        delay_us(2);
        output_low(SCK);
        v = v << 1;
        if (input(SDI))
            v = v | 1;
        delay_us(2);
    }

    return (v);
}

//******************************************************************************
//* getData (Retrieve PRESSURE and TEMPERATURE data from MS5534A)
//******************************************************************************
#separate
long get_data( int reg_sel)
{
    long d1;
    int error, pulse = 2;

    if (reg_sel == PRESSURE)
    {
        SerialSendLsbFirst (0x2F, 8);
        SerialSendLsbFirst (0x00, 2);
    }
    else
    {
        SerialSendLsbFirst (0x4F, 8);
        SerialSendLsbFirst (0x00, 3);
        pulse = 1;
    }

    error = 0;
    if (input(SDI) == FALSE)
        error = 1;

    SerialSendLsbFirst (0x00, pulse);

    if (!error)
        error = waitOnDoutFall();

    if (!error)
        d1 = SerialGet16();
    else
        d1 = 0;

    if (reg_sel == PRESSURE)
        SerialSendLsbFirst (0x00, 1);

    //if (error_pt != 0)
    //  *error_pt = error;

    return(d1);
}

//******************************************************************************
//* WaitOnDoutFall (Wait for ADC complete delay signal on data out)
//******************************************************************************
#separate
long waitOnDoutFall (void)
{
    int             cnt = 0;
    short           working = TRUE;
    long            error = 0;

    while (working)
    {
        working = input(SDI);
        cnt++;
        delay_ms(1);
        if (cnt>=50)
        {
            working = FALSE;
            error = 1;
        }
    }

    return (error);
}

//******************************************************************************
//* ConvertWtoC (Extract the six coeffcients from the 4 cal words retrieved
//* previously)
//******************************************************************************
#separate
long ConvertWtoC5534 (int ix, long W1, long W2, long W3, long W4)
{
    long c = 0;
    long x, y;

    switch(ix)
    {
        case 0:
                c = (W1 >> 1) &  0x7FFF;
                break;
        case 1:
                x = (W3 << 6) &  0x0FC0;
                y = W4 &         0x003F;
                c = x | y;
                break;
        case 2:
                c = (W4 >> 6) &  0x03FF;
                break;
        case 3:
                c = (W3 >> 6) &  0x03FF;
                break;
        case 4:
                x = (W1 << 10) & 0x0400;
                y = (W2 >> 6)  & 0x03FF;
                c = x | y;
                break;
        case 5:
                c = W2 &         0x003F;
                break;
    }

    return (c);
}